from enum import IntEnum


class AudioGenres(IntEnum):
    rock = 1
    pop = 2
    rap = 3
    hip_hop = 3
    easy_listening = 4
    house = 5
    dance = 5
    instrumental = 6
    metal = 7
    alternative = 21
    dubstep = 8
    jazz = 1001
    blues = 1001
    drum = 10
    bass = 10
    trance = 11
    chanson = 12
    ethnic = 13
    acoustic = 14
    vocal = 14
    reggat = 15
    classical = 16
    indie_pop = 17
    other = 18
    speech = 19
    electropop = 22
    disco = 22
